import chalk from 'chalk'
import spawn from 'cross-spawn'
import path from 'path'
import {
    copyFile, getInput, checkYarn, replaceFileAppName, checkPath
} from '../utils'

const { join } = path
const INIT_APP_NAME = 'ttk-app-init-new-demo';
const INIT_APP_PRE_NAME = 'app-test';

async function createApp(path) {
    await checkYarn()
    if (typeof (path) != 'string') {
        console.log(chalk.yellowBright('你没有输入创建的app名字！'))
        path = await getInput('请输入创建的app名称：')
    }
  
    console.log('检查该路径下是否已经存在app...')
    // 默认在src/jsonApps文件夹下创建
    path = `src/jsonApps/${path}`
    await checkPath(path)

    const cloneResult = await spawn.sync('yarn', ['add', INIT_APP_NAME], { cwd: join(process.cwd()), stdio: 'inherit' })
    if (cloneResult.error || cloneResult.status != 0) {
        console.log(chalk.redBright(cloneResult.error))
        console.log(chalk.redBright(`下载${app}失败！`))
        return process.exit()
    }
    // 复制文件
    await copyFile(
        `${path}`,
        [
            `./node_modules/${INIT_APP_NAME}/**`,
            `!./node_modules/${INIT_APP_NAME}/package.json`,
            `!./node_modules/${INIT_APP_NAME}/README.md`,
            `!./node_modules/${INIT_APP_NAME}/node_modules`
        ]
    )
    console.log(chalk.greenBright('复制文件成功！！！'))

    const arr = path.split('/')
    const name = arr[arr.length - 1]
    if (name == INIT_APP_NAME) {
        return;
    }
    await replaceFileAppName(`./${path}/index.js`, name, INIT_APP_PRE_NAME)
    await replaceFileAppName(`./${path}/data.js`, name, INIT_APP_PRE_NAME)
    await replaceFileAppName(`./${path}/style.less`, name, INIT_APP_PRE_NAME)
    console.log(chalk.greenBright('完成'))
    process.exit()
}

export default createApp